@@ -14,8 +14,7 @@
 #    under the License.
 
 from neutron.db import common_db_mixin as base_db
-from neutron.plugins.common import constants as p_const
-import neutron_lib.constants as libconstants
+from neutron_lib import constants as nl_constants
 from neutron_lib.db import model_base
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -103,7 +102,6 @@ class FirewallPolicyRuleAssociation(model_base.BASEV2):
                                                ondelete="CASCADE"),
                                  primary_key=True)
     position = sa.Column(sa.Integer)
-    firewall_rule = orm.relationship('FirewallRuleV2')
 
 
 class FirewallPolicy(model_base.BASEV2, model_base.HasId, HasName,
@@ -141,22 +139,27 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         except exc.NoResultFound:
             raise fw_ext.FirewallRuleNotFound(firewall_rule_id=id)
 
-    def _validate_fwr_protocol_parameters(self, fwr):
-        protocol = fwr['protocol']
-        if protocol not in (libconstants.PROTO_NAME_TCP,
-                            libconstants.PROTO_NAME_UDP):
-            if fwr['source_port'] or fwr['destination_port']:
+    def _validate_fwr_protocol_parameters(self, fwr, fwr_db=None):
+        protocol = fwr.get('protocol', None)
+        if fwr_db and not protocol:
+            protocol = fwr_db.protocol
+        if protocol not in (nl_constants.PROTO_NAME_TCP,
+                            nl_constants.PROTO_NAME_UDP):
+            if (fwr.get('source_port', None) or
+                    fwr.get('destination_port', None)):
                 raise fw_ext.FirewallRuleInvalidICMPParameter(
                     param="Source, destination port")
 
-    def _validate_fwr_src_dst_ip_version(self, fwr):
+    def _validate_fwr_src_dst_ip_version(self, fwr, fwr_db=None):
         src_version = dst_version = None
-        if fwr['source_ip_address']:
+        if fwr.get('source_ip_address', None):
             src_version = netaddr.IPNetwork(fwr['source_ip_address']).version
-        if fwr['destination_ip_address']:
+        if fwr.get('destination_ip_address', None):
             dst_version = netaddr.IPNetwork(
                 fwr['destination_ip_address']).version
-        rule_ip_version = fwr['ip_version']
+        rule_ip_version = fwr.get('ip_version', None)
+        if not rule_ip_version and fwr_db:
+            rule_ip_version = fwr_db.ip_version
         if ((src_version and src_version != rule_ip_version) or
                 (dst_version and dst_version != rule_ip_version)):
             raise fw_ext.FirewallIpAddressConflict()
@@ -220,6 +223,9 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         return self._fields(res, fields)
 
     def _make_firewall_group_dict(self, firewall_group, fields=None):
+        fwg_ports = [
+            port_assoc.port_id for port_assoc in firewall_group['ports']
+        ]
         res = {'id': firewall_group['id'],
                'tenant_id': firewall_group['tenant_id'],
                'name': firewall_group['name'],
@@ -230,6 +236,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                'egress_firewall_policy_id':
                    firewall_group['egress_firewall_policy_id'],
                'admin_state_up': firewall_group['admin_state_up'],
+               'ports': fwg_ports,
                'status': firewall_group['status']}
         return self._fields(res, fields)
 
@@ -299,7 +306,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             self._get_policy_rule_association_query(
                 context, firewall_policy_id, firewall_rule_id).one()
             raise fw_ext.FirewallRuleAlreadyAssociated(
-                firewall_rule_id=firewall_rule_id, firewall_policy_id=id)
+                firewall_rule_id=firewall_rule_id,
+                firewall_policy_id=firewall_policy_id)
         except exc.NoResultFound:
             return
 
@@ -313,7 +321,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 context, firewall_policy_id, firewall_rule_id).one()
         except exc.NoResultFound:
             raise fw_ext.FirewallRuleNotAssociatedWithPolicy(
-                firewall_rule_id=firewall_rule_id, firewall_policy_id=id)
+                firewall_rule_id=firewall_rule_id,
+                firewall_policy_id=firewall_policy_id)
 
     def create_firewall_rule(self, context, firewall_rule):
         LOG.debug("create_firewall_rule() called")
@@ -351,6 +360,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         LOG.debug("update_firewall_rule() called")
         fwr = firewall_rule['firewall_rule']
         fwr_db = self._get_firewall_rule(context, id)
+        self._validate_fwr_protocol_parameters(fwr, fwr_db=fwr_db)
+        self._validate_fwr_src_dst_ip_version(fwr, fwr_db=fwr_db)
         if 'source_port' in fwr:
             src_port_min, src_port_max = self._get_min_max_ports_from_range(
                 fwr['source_port'])
@@ -575,6 +586,12 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         fwp_db = firewall_policy_db
         with context.session.begin(subtransactions=True):
             if not rule_id_list:
+                for rule_id in [rule_assoc.firewall_rule_id
+                    for rule_assoc in fwp_db['rule_associations']]:
+                    fwpra_db = self._get_policy_rule_association(
+                        context, fwp_db['id'], rule_id)
+                    fwp_db.rule_associations.remove(fwpra_db)
+                    context.session.delete(fwpra_db)
                 fwp_db.rule_associations = []
                 return
             # We will first check if the new list of rules is valid
@@ -586,6 +603,9 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             self._delete_rules_in_policy(context, fwp_db['id'])
             # and add in the new association
             self._set_rules_in_policy_rule_assoc(context, fwp_db, fwp)
+            # we need care about the associations related with this policy
+            # and its rules only.
+            filters['firewall_policy_id'] = [fwp_db['id']]
             rules_in_fpol_rul_db = self._get_collection_query(
                 context,
                 FirewallPolicyRuleAssociation,
@@ -726,8 +746,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
     def create_firewall_group(self, context, firewall_group, status=None):
         fwg = firewall_group['firewall_group']
         if not status:
-            status = (p_const.CREATED if cfg.CONF.router_distributed
-                      else p_const.PENDING_CREATE)
+            status = (nl_constants.CREATED if cfg.CONF.router_distributed
+                      else nl_constants.PENDING_CREATE)
         with context.session.begin(subtransactions=True):
             self._validate_fwg_parameters(context, fwg, fwg['tenant_id'])
             fwg_db = FirewallGroup(id=uuidutils.generate_uuid(),
