@@ -13,11 +13,11 @@
 # under the License.
 
 import abc
-from neutron.api import extensions
-from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import resource_helper
 from neutron.services import service_base
 from neutron_lib.api import converters
+from neutron_lib.api import extensions
+from neutron_lib.db import constants as nl_db_constants
 from neutron_lib import exceptions as nexception
 import six
 
@@ -51,7 +51,8 @@ class FirewallGroupPortInvalid(nexception.Conflict):
 
 
 class FirewallGroupPortInvalidProject(nexception.Conflict):
-    message = _("Firewall Group %(port_id)s in invalid Project")
+    message = _("Operation cannot be performed as port %(port_id)s "
+                "is in an invalid project %(tenant_id)s.")
 
 
 class FirewallGroupPortInUse(nexception.InUse):
@@ -113,7 +114,7 @@ class FirewallRuleInUse(nexception.InUse):
 
 class FirewallRuleNotAssociatedWithPolicy(nexception.InvalidInput):
     message = _("Firewall Rule %(firewall_rule_id)s is not associated "
-                " with Firewall Policy %(firewall_policy_id)s.")
+                "with Firewall Policy %(firewall_policy_id)s.")
 
 
 class FirewallRuleInvalidProtocol(nexception.InvalidInput):
@@ -190,29 +191,30 @@ RESOURCE_ATTRIBUTE_MAP = {
                'is_visible': True, 'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'required_by_policy': True,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {'type:string':
+                                   nl_db_constants.UUID_FIELD_SIZE},
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': nl_db_constants.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string':
-                                     attr.DESCRIPTION_MAX_LEN},
+                                     nl_db_constants.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'firewall_policy_id': {'allow_post': False, 'allow_put': False,
                                'validate': {'type:uuid_or_none': None},
                                'is_visible': True},
         'public': {'allow_post': True, 'allow_put': True,
-                   'default': False, 'convert_to': attr.convert_to_boolean,
-                   'is_visible': True, 'required_by_policy': True,
-                   'enforce_policy': True},
+                   'default': False, 'is_visible': True,
+                   'convert_to': converters.convert_to_boolean,
+                   'required_by_policy': True, 'enforce_policy': True},
         'protocol': {'allow_post': True, 'allow_put': True,
                      'is_visible': True, 'default': None,
                      'convert_to': fwaas_v1.convert_protocol,
                      'validate': {'type:values':
                                   fwaas_v1.fw_valid_protocol_values}},
         'ip_version': {'allow_post': True, 'allow_put': True,
-                       'default': 4, 'convert_to': attr.convert_to_int,
+                       'default': 4, 'convert_to': converters.convert_to_int,
                        'validate': {'type:values': [4, 6]},
                        'is_visible': True},
         'source_ip_address': {'allow_post': True, 'allow_put': True,
@@ -238,8 +240,8 @@ RESOURCE_ATTRIBUTE_MAP = {
                                 fwaas_v1.fw_valid_action_values},
                    'is_visible': True, 'default': 'deny'},
         'enabled': {'allow_post': True, 'allow_put': True,
-                    'default': True, 'convert_to': attr.convert_to_boolean,
-                    'is_visible': True},
+                    'convert_to': converters.convert_to_boolean,
+                    'default': True, 'is_visible': True},
     },
     'firewall_groups': {
         'id': {'allow_post': False, 'allow_put': False,
@@ -247,28 +249,29 @@ RESOURCE_ATTRIBUTE_MAP = {
                'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': nl_db_constants.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string':
-                                     attr.DESCRIPTION_MAX_LEN},
+                                     nl_db_constants.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True, 'is_visible': True,
                            'convert_to': converters.convert_to_boolean},
         'status': {'allow_post': False, 'allow_put': False,
                    'is_visible': True},
-        'public': {'allow_post': True, 'allow_put': True,
-                   'default': False, 'convert_to': attr.convert_to_boolean,
+        'public': {'allow_post': True, 'allow_put': True, 'default': False,
+                   'convert_to': converters.convert_to_boolean,
                    'is_visible': True, 'required_by_policy': True,
                    'enforce_policy': True},
         'ports': {'allow_post': True, 'allow_put': True,
                   'validate': {'type:uuid_list': None},
-                  'convert_to': attr.convert_none_to_empty_list,
+                  'convert_to': converters.convert_none_to_empty_list,
                   'default': None, 'is_visible': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'required_by_policy': True,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {'type:string':
+                                   nl_db_constants.UUID_FIELD_SIZE},
                       'is_visible': True},
         'ingress_firewall_policy_id': {'allow_post': True,
                                        'allow_put': True,
@@ -288,25 +291,26 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'required_by_policy': True,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {'type:string':
+                                   nl_db_constants.UUID_FIELD_SIZE},
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': nl_db_constants.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string':
-                                     attr.DESCRIPTION_MAX_LEN},
+                                     nl_db_constants.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
-        'public': {'allow_post': True, 'allow_put': True,
-                   'default': False, 'convert_to': attr.convert_to_boolean,
+        'public': {'allow_post': True, 'allow_put': True, 'default': False,
+                   'convert_to': converters.convert_to_boolean,
                    'is_visible': True, 'required_by_policy': True,
                    'enforce_policy': True},
         'firewall_rules': {'allow_post': True, 'allow_put': True,
                            'validate': {'type:uuid_list': None},
-                           'convert_to': attr.convert_none_to_empty_list,
+                           'convert_to': converters.convert_none_to_empty_list,
                            'default': None, 'is_visible': True},
-        'audited': {'allow_post': True, 'allow_put': True,
-                    'default': False, 'convert_to': attr.convert_to_boolean,
+        'audited': {'allow_post': True, 'allow_put': True, 'default': False,
+                    'convert_to': converters.convert_to_boolean,
                     'is_visible': True},
 
     },
@@ -336,7 +340,6 @@ class Firewall_v2(extensions.ExtensionDescriptor):
         special_mappings = {'firewall_policies': 'firewall_policy'}
         plural_mappings = resource_helper.build_plural_mappings(
             special_mappings, RESOURCE_ATTRIBUTE_MAP)
-        attr.PLURALS.update(plural_mappings)
         action_map = {'firewall_policy': {'insert_rule': 'PUT',
                                           'remove_rule': 'PUT'}}
         return resource_helper.build_resource_info(plural_mappings,
