@@ -12,16 +12,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.plugins import directory
+
 from neutron.common import rpc as n_rpc
 from neutron import context as neutron_context
-from neutron import manager
-from neutron.plugins.common import constants as n_const
+from neutron_lib import constants as nl_constants
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 
-from neutron_fwaas._i18n import _LI
-from neutron_fwaas.common import fwaas_constants as f_const
+from neutron_fwaas.common import fwaas_constants
 from neutron_fwaas.db.firewall.v2 import firewall_db_v2
 from neutron_fwaas.extensions import firewall_v2 as fw_ext
 
@@ -68,40 +68,37 @@ class FirewallCallbacks(object):
         LOG.debug("Setting firewall_group %s to status: %s" % (
             fwg_id, status))
         # Sanitize status first
-        if status in (n_const.ACTIVE, n_const.DOWN, n_const.INACTIVE):
+        if status in (nl_constants.ACTIVE, nl_constants.DOWN,
+                      nl_constants.INACTIVE):
             to_update = status
         else:
-            to_update = n_const.ERROR
+            to_update = nl_constants.ERROR
         # ignore changing status if firewall_group expects to be deleted
         # That case means that while some pending operation has been
         # performed on the backend, neutron server received delete request
         # and changed firewall status to PENDING_DELETE
         updated = self.plugin.update_firewall_group_status(
-            context, fwg_id, to_update, not_in=(n_const.PENDING_DELETE,))
+            context, fwg_id, to_update, not_in=(nl_constants.PENDING_DELETE,))
         if updated:
             LOG.debug("firewall %s status set: %s" % (fwg_id, to_update))
-        return updated and to_update != n_const.ERROR
+        return updated and to_update != nl_constants.ERROR
 
     def firewall_group_deleted(self, context, fwg_id, **kwargs):
         """Agent uses this to indicate firewall is deleted."""
         LOG.debug("firewall_group_deleted() called")
-        try:
-            with context.session.begin(subtransactions=True):
-                fwg_db = self.plugin._get_firewall_group(context, fwg_id)
-                # allow to delete firewalls in ERROR state
-                if fwg_db.status in (n_const.PENDING_DELETE, n_const.ERROR):
-                    self.plugin.delete_db_firewall_group_object(context,
-                                                                fwg_id)
-                    return True
-                else:
-                    LOG.warning(('Firewall %(fwg)s unexpectedly deleted by '
-                                 'agent, status was %(status)s'),
-                                {'fwg': fwg_id, 'status': fwg_db.status})
-                    fwg_db.update({"status": n_const.ERROR})
-                    return False
-        except fw_ext.FirewallGroupNotFound:
-            LOG.info(_LI('Firewall group %s already deleted'), fwg_id)
-            return True
+        with context.session.begin(subtransactions=True):
+            fwg_db = self.plugin._get_firewall_group(context, fwg_id)
+            # allow to delete firewalls in ERROR state
+            if fwg_db.status in (nl_constants.PENDING_DELETE,
+                                 nl_constants.ERROR):
+                self.plugin.delete_db_firewall_group_object(context, fwg_id)
+                return True
+            else:
+                LOG.warning(('Firewall %(fwg)s unexpectedly deleted by '
+                             'agent, status was %(status)s'),
+                            {'fwg': fwg_id, 'status': fwg_db.status})
+                fwg_db.update({"status": nl_constants.ERROR})
+                return False
 
     def get_firewall_groups_for_project(self, context, **kwargs):
         """Gets all firewall_groups and rules on a project."""
@@ -110,7 +107,7 @@ class FirewallCallbacks(object):
         for fwg in self.plugin.get_firewall_groups(context):
             fwg_with_rules = self.plugin._make_firewall_group_dict_with_rules(
                 context, fwg['id'])
-            if fwg['status'] == n_const.PENDING_DELETE:
+            if fwg['status'] == nl_constants.PENDING_DELETE:
                 fwg_with_rules['add-port-ids'] = []
                 fwg_with_rules['del-port-ids'] = (
                     self.plugin._get_ports_in_firewall_group(context,
@@ -148,24 +145,25 @@ class FirewallPluginV2(
         self.start_rpc_listeners()
 
         self.agent_rpc = FirewallAgentApi(
-            f_const.FW_AGENT,
+            fwaas_constants.FW_AGENT,
             cfg.CONF.host
         )
 
     @property
     def _core_plugin(self):
-        return manager.NeutronManager.get_plugin()
+        return directory.get_plugin()
 
     def start_rpc_listeners(self):
         self.endpoints = [FirewallCallbacks(self)]
 
         self.conn = n_rpc.create_connection()
         self.conn.create_consumer(
-            f_const.FIREWALL_PLUGIN, self.endpoints, fanout=False)
+            fwaas_constants.FIREWALL_PLUGIN, self.endpoints, fanout=False)
         return self.conn.consume_in_threads()
 
     def _rpc_update_firewall_group(self, context, fwg_id):
-        status_update = {"firewall_group": {"status": n_const.PENDING_UPDATE}}
+        status_update = {"firewall_group": {"status":
+                         nl_constants.PENDING_UPDATE}}
         super(FirewallPluginV2, self).update_firewall_group(
             context, fwg_id, status_update)
         fwg_with_rules = self._make_firewall_group_dict_with_rules(context,
@@ -187,9 +185,9 @@ class FirewallPluginV2(
 
     def _ensure_update_firewall_group(self, context, fwg_id):
         fwg = self.get_firewall_group(context, fwg_id)
-        if fwg['status'] in [n_const.PENDING_CREATE,
-                             n_const.PENDING_UPDATE,
-                             n_const.PENDING_DELETE]:
+        if fwg['status'] in [nl_constants.PENDING_CREATE,
+                             nl_constants.PENDING_UPDATE,
+                             nl_constants.PENDING_DELETE]:
             raise fw_ext.FirewallGroupInPendingState(firewall_id=fwg_id,
                                                 pending_state=fwg['status'])
 
@@ -214,9 +212,22 @@ class FirewallPluginV2(
             if port_db['device_owner'] != "network:router_interface":
                 raise fw_ext.FirewallGroupPortInvalid(port_id=port_id)
             if port_db['tenant_id'] != tenant_id:
-                raise fw_ext.FirewallGroupPortInvalidProject(port_id=port_id)
+                raise fw_ext.FirewallGroupPortInvalidProject(
+                    port_id=port_id, tenant_id=port_db['tenant_id'])
         return
 
+    def _check_no_need_pending(self, context, fwg_id, fwg_body):
+        fwg_db = self._get_firewall_group(context, fwg_id)
+        fwp_req_in = fwg_body.get('ingress_firewall_policy_id', None)
+        fwp_req_eg = fwg_body.get('egress_firewall_policy_id', None)
+
+        if ((not fwg_db.ingress_firewall_policy_id and
+                fwp_req_in is fwg_db.ingress_firewall_policy_id) and
+                (not fwg_db.egress_firewall_policy_id and
+                    fwp_req_eg is fwg_db.ingress_firewall_policy_id)):
+            return True
+        return False
+
     def create_firewall_group(self, context, firewall_group):
         LOG.debug("create_firewall_group() called")
         fwgrp = firewall_group['firewall_group']
@@ -224,7 +235,7 @@ class FirewallPluginV2(
         if not fwg_ports:
             # no messaging to agent needed, and fw needs to go
             # to INACTIVE(no associated ports) state.
-            status = n_const.INACTIVE
+            status = nl_constants.INACTIVE
             fwg = super(FirewallPluginV2, self).create_firewall_group(
                 context, firewall_group, status)
             fwg['ports'] = []
@@ -239,7 +250,7 @@ class FirewallPluginV2(
             if (not fwgrp['ingress_firewall_policy_id'] and
                 not fwgrp['egress_firewall_policy_id']):
                 # No policy configured
-                status = n_const.INACTIVE
+                status = nl_constants.INACTIVE
                 fwg = super(FirewallPluginV2, self).create_firewall_group(
                     context, firewall_group, status)
                 return fwg
@@ -271,6 +282,8 @@ class FirewallPluginV2(
                 # This indicates that user is indicating no ports.
                 fwg_new_ports = []
             else:
+                self._validate_ports_for_firewall_group(
+                    context, context.tenant_id, fwg_ports)
                 self._validate_if_firewall_group_on_ports(
                     context, fwg_ports, id)
                 fwg_new_ports = fwg_ports
@@ -279,16 +292,24 @@ class FirewallPluginV2(
             # existing ports.
             fwg_new_ports = self._get_ports_in_firewall_group(context, id)
 
-        if not fwg_new_ports and not fwg_current_ports:
+        if ((not fwg_new_ports and not fwg_current_ports) or
+            self._check_no_need_pending(context,
+                                        id, firewall_group['firewall_group'])):
             # no messaging to agent needed, and we need to continue
             # in INACTIVE state
-            firewall_group['firewall_group']['status'] = n_const.INACTIVE
+            firewall_group['firewall_group']['status'] = nl_constants.INACTIVE
             fwg = super(FirewallPluginV2, self).update_firewall_group(
                 context, id, firewall_group)
-            fwg['ports'] = []
+            if fwg_new_ports:
+                fwg['ports'] = fwg_new_ports
+            elif not fwg_new_ports and fwg_current_ports:
+                fwg['ports'] = fwg_current_ports
+            else:
+                fwg['ports'] = []
             return fwg
         else:
-            firewall_group['firewall_group']['status'] = n_const.PENDING_UPDATE
+            firewall_group['firewall_group']['status'] = (nl_constants.
+                                                          PENDING_UPDATE)
             fwg = super(FirewallPluginV2, self).update_firewall_group(
                 context, id, firewall_group)
             fwg['ports'] = fwg_new_ports
@@ -327,7 +348,8 @@ class FirewallPluginV2(
             # no ports, no need to talk to the agent
             self.delete_db_firewall_group_object(context, id)
         else:
-            status = {"firewall_group": {"status": n_const.PENDING_DELETE}}
+            status = {"firewall_group":
+                     {"status": nl_constants.PENDING_DELETE}}
             super(FirewallPluginV2, self).update_firewall_group(
                 context, id, status)
             # Reflect state change in fw_with_rules
