@@ -2,7 +2,7 @@
 
 set -ex
 
-# Below variables are set to exceute this script
+# Below variables are set to execute this script
 IS_GATE=${IS_GATE:-False}
 INSTALL_MYSQL_ONLY=${INSTALL_MYSQL_ONLY:-False}
 
@@ -23,6 +23,7 @@ source $TOP_DIR/functions
 source $TOP_DIR/inc/meta-config
 source $TOP_DIR/stackrc
 source $TOP_DIR/lib/database
+source $TOP_DIR/localrc
 
 # Install_databases [install_pg]
 # Tweak the script accordingly if we need psql in future
