@@ -19,10 +19,7 @@ import mock
 from neutron.api import extensions as api_ext
 from neutron.common import config
 from neutron import context
-import neutron.extensions as nextensions
 from neutron.extensions import l3
-from neutron import manager
-from neutron.plugins.common import constants
 from oslo_config import cfg
 from oslo_utils import importutils
 from oslo_utils import uuidutils
@@ -34,13 +31,15 @@ from neutron_fwaas import extensions
 from neutron_fwaas.extensions import firewall
 from neutron_fwaas.services.firewall import fwaas_plugin
 from neutron_fwaas.tests import base
+from neutron_lib import constants as nl_constants
+from neutron_lib.plugins import directory
 
 DB_FW_PLUGIN_KLASS = (
     "neutron_fwaas.db.firewall.firewall_db.Firewall_db_mixin"
 )
 FWAAS_PLUGIN = 'neutron_fwaas.services.firewall.fwaas_plugin'
 DELETEFW_PATH = FWAAS_PLUGIN + '.FirewallAgentApi.delete_firewall'
-extensions_path = ':'.join(extensions.__path__ + nextensions.__path__)
+extensions_path = ':'.join(extensions.__path__)
 DESCRIPTION = 'default description'
 SHARED = True
 PROTOCOL = 'tcp'
@@ -68,7 +67,7 @@ class FakeAgentApi(fwaas_plugin.FirewallCallbacks):
         pass
 
     def delete_firewall(self, context, firewall, **kwargs):
-        self.plugin = manager.NeutronManager.get_service_plugins()['FIREWALL']
+        self.plugin = directory.get_plugin('FIREWALL')
         self.firewall_deleted(context, firewall['id'], **kwargs)
 
 
@@ -97,7 +96,7 @@ class FirewallPluginDbTestCase(base.NeutronDbPluginV2TestCase):
             self.plugin = importutils.import_object(fw_plugin)
             ext_mgr = api_ext.PluginAwareExtensionManager(
                 extensions_path,
-                {constants.FIREWALL: self.plugin}
+                {'FIREWALL': self.plugin}
             )
             app = config.load_paste_app('extensions_test_app')
             self.ext_api = api_ext.ExtensionMiddleware(app, ext_mgr=ext_mgr)
@@ -114,9 +113,14 @@ class FirewallPluginDbTestCase(base.NeutronDbPluginV2TestCase):
                          neutron_context=neutron_context,
                          query_params=query_params)
         resource = resource.replace('-', '_')
-        self.assertEqual(
-            sorted([i[resource]['id'] for i in items]),
-            sorted([i['id'] for i in res[resource_plural]]))
+        if query_params is None or ('fields' not in query_params):
+            self.assertEqual(
+                sorted([i[resource]['id'] for i in items]),
+                sorted([i['id'] for i in res[resource_plural]]))
+        else:
+            self.assertEqual(
+                sorted([i for i in items]),
+                sorted([i for i in res[resource_plural]]))
 
     def _get_test_firewall_rule_attrs(self, name='firewall_rule1'):
         attrs = {'name': name,
@@ -816,6 +820,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
         attrs['destination_port'] = '30:40'
         with self.firewall_rule() as fwr:
             data = {'firewall_rule': {'name': name,
+                                      'protocol': PROTOCOL,
                                       'source_port': '10:20',
                                       'destination_port': '30:40'}}
             req = self.new_update_request('firewall_rules', data,
@@ -829,6 +834,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
         attrs['destination_port'] = '80'
         with self.firewall_rule() as fwr:
             data = {'firewall_rule': {'name': name,
+                                      'protocol': PROTOCOL,
                                       'source_port': 10000,
                                       'destination_port': 80}}
             req = self.new_update_request('firewall_rules', data,
@@ -842,6 +848,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
         attrs['destination_port'] = '80'
         with self.firewall_rule() as fwr:
             data = {'firewall_rule': {'name': name,
+                                      'protocol': PROTOCOL,
                                       'source_port': '10000',
                                       'destination_port': '80'}}
             req = self.new_update_request('firewall_rules', data,
@@ -912,6 +919,17 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
             res = req.get_response(self.ext_api)
             self.assertEqual(200, res.status_int)
 
+    def test_update_firewall_rule_icmp_with_port(self):
+        with self.firewall_rule(source_port=None,
+                                destination_port=None,
+                                protocol=None) as fwr:
+            data = {'firewall_rule': {'destination_port': 80,
+                                      'protocol': 'icmp'}}
+            req = self.new_update_request('firewall_rules', data,
+                                          fwr['firewall_rule']['id'])
+            res = req.get_response(self.ext_api)
+            self.assertEqual(400, res.status_int)
+
     def test_update_firewall_rule_with_policy_associated(self):
         name = "new_firewall_rule1"
         attrs = self._get_test_firewall_rule_attrs(name)
@@ -1158,7 +1176,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
                                context=ctx) as firewall:
                 fw_id = firewall['firewall']['id']
                 fw_db = self.plugin._get_firewall(ctx, fw_id)
-                fw_db['status'] = constants.ACTIVE
+                fw_db['status'] = nl_constants.ACTIVE
                 # update firewall from fwp1 to fwp2(different tenant)
                 data = {'firewall': {'firewall_policy_id': fwp2_id}}
                 req = self.new_update_request('firewalls', data, fw_id,
@@ -1378,6 +1396,24 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
                                   expected_code=webob.exc.HTTPOk.code,
                                   expected_body=attrs)
 
+    def test_remove_rule_and_not_associated(self):
+        with self.firewall_rule(name='fwr0') as fwr1, \
+            self.firewall_rule(name='fwr2') as fwr2:
+            associated = fwr1['firewall_rule']['id']
+            with self.firewall_policy(
+                name='firewall_policy2', firewall_rules=[associated]) as fwp:
+                fwp_id = fwp['firewall_policy']['id']
+                not_associated = fwr2['firewall_rule']['id']
+                msg = "Firewall Rule {0} is not associated with " \
+                      "Firewall Policy {1}.".format(not_associated, fwp_id)
+                result = self._rule_action(
+                    'remove', fwp_id, not_associated,
+                    insert_before=None,
+                    insert_after=None,
+                    expected_code=webob.exc.HTTPBadRequest.code,
+                    body_data={'firewall_rule_id': not_associated})
+                self.assertEqual(msg, result['NeutronError']['message'])
+
     def test_remove_rule_from_policy(self):
         attrs = self._get_test_firewall_policy_attrs()
         attrs['audited'] = False
@@ -1442,25 +1478,21 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
                                   body_data={'firewall_rule_id': None})
 
     def test_check_router_has_no_firewall_raises(self):
-        with mock.patch.object(
-            manager.NeutronManager, 'get_service_plugins') as sp:
-            fw_plugin = mock.Mock()
-            sp.return_value = {'FIREWALL': fw_plugin}
-            fw_plugin.get_firewalls.return_value = [mock.ANY]
-            kwargs = {
-                'context': mock.ANY,
-                'router': {'id': 'foo_id', 'tenant_id': 'foo_tenant'}
-            }
-            self.assertRaises(
-                l3.RouterInUse,
-                fdb.migration_callback,
-                'router', 'before_event', mock.ANY,
-                **kwargs)
+        fw_plugin = mock.Mock()
+        directory.add_plugin('FIREWALL', fw_plugin)
+        fw_plugin.get_firewalls.return_value = [mock.ANY]
+        kwargs = {
+            'context': mock.ANY,
+            'router': {'id': 'foo_id', 'tenant_id': 'foo_tenant'}
+        }
+        self.assertRaises(
+            l3.RouterInUse,
+            fdb.migration_callback,
+            'router', 'before_event', mock.ANY,
+            **kwargs)
 
     def test_check_router_has_no_firewall_passes(self):
-        with mock.patch.object(manager.NeutronManager,
-                               'get_service_plugins',
-                               return_value={}):
+        with mock.patch.object(directory, 'get_plugin', return_value=None):
             kwargs = {'context': mock.ANY, 'router': mock.ANY}
             self.assertIsNone(fdb.migration_callback(
                 mock.ANY, mock.ANY, mock.ANY, **kwargs))
