@@ -18,8 +18,6 @@ import testtools
 
 from neutron.api.v2 import attributes as attr
 from neutron import context
-from neutron import manager
-from neutron.plugins.common import constants as const
 from neutron.tests import fake_notifier
 from neutron.tests.unit.extensions import test_l3 as test_l3_plugin
 from oslo_config import cfg
@@ -33,6 +31,8 @@ from neutron_fwaas.services.firewall import fwaas_plugin
 from neutron_fwaas.tests import base
 from neutron_fwaas.tests.unit.db.firewall import (
     test_firewall_db as test_db_firewall)
+from neutron_lib import constants as nl_constants
+from neutron_lib.plugins import directory
 
 extensions_path = neutron_fwaas.extensions.__path__[0]
 
@@ -74,6 +74,7 @@ class TestFirewallRouterInsertionBase(
         self.saved_attr_map = {}
         for resource, attrs in six.iteritems(attr.RESOURCE_ATTRIBUTE_MAP):
             self.saved_attr_map[resource] = attrs.copy()
+        self.addCleanup(self.restore_attribute_map)
         if not fw_plugin:
             fw_plugin = FW_PLUGIN_KLASS
         service_plugins = {'l3_plugin_name': l3_plugin,
@@ -86,10 +87,8 @@ class TestFirewallRouterInsertionBase(
 
         self.setup_notification_driver()
 
-        self.l3_plugin = manager.NeutronManager.get_service_plugins().get(
-            const.L3_ROUTER_NAT)
-        self.plugin = manager.NeutronManager.get_service_plugins().get(
-            const.FIREWALL)
+        self.l3_plugin = directory.get_plugin(nl_constants.L3)
+        self.plugin = directory.get_plugin('FIREWALL')
         self.callbacks = self.plugin.endpoints[0]
 
     def restore_attribute_map(self):
@@ -98,10 +97,6 @@ class TestFirewallRouterInsertionBase(
         # Restore the original RESOURCE_ATTRIBUTE_MAP
         attr.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map
 
-    def tearDown(self):
-        self.restore_attribute_map()
-        super(TestFirewallRouterInsertionBase, self).tearDown()
-
     def _create_firewall(self, fmt, name, description, firewall_policy_id=None,
                          admin_state_up=True, expected_res_status=None,
                          **kwargs):
@@ -146,15 +141,15 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
             ) as fw:
                 fw_id = fw['firewall']['id']
                 res = self.callbacks.set_firewall_status(ctx, fw_id,
-                                                         const.ACTIVE,
+                                                         nl_constants.ACTIVE,
                                                          host='dummy')
                 fw_db = self.plugin.get_firewall(ctx, fw_id)
-                self.assertEqual(const.ACTIVE, fw_db['status'])
+                self.assertEqual(nl_constants.ACTIVE, fw_db['status'])
                 self.assertTrue(res)
                 res = self.callbacks.set_firewall_status(ctx, fw_id,
-                                                         const.ERROR)
+                                                         nl_constants.ERROR)
                 fw_db = self.plugin.get_firewall(ctx, fw_id)
-                self.assertEqual(const.ERROR, fw_db['status'])
+                self.assertEqual(nl_constants.ERROR, fw_db['status'])
                 self.assertFalse(res)
 
     def test_set_firewall_status_pending_delete(self):
@@ -167,13 +162,13 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
             ) as fw:
                 fw_id = fw['firewall']['id']
                 fw_db = self.plugin._get_firewall(ctx, fw_id)
-                fw_db['status'] = const.PENDING_DELETE
+                fw_db['status'] = nl_constants.PENDING_DELETE
                 ctx.session.flush()
                 res = self.callbacks.set_firewall_status(ctx, fw_id,
-                                                         const.ACTIVE,
+                                                         nl_constants.ACTIVE,
                                                          host='dummy')
                 fw_db = self.plugin.get_firewall(ctx, fw_id)
-                self.assertEqual(const.PENDING_DELETE, fw_db['status'])
+                self.assertEqual(nl_constants.PENDING_DELETE, fw_db['status'])
                 self.assertFalse(res)
 
     def test_firewall_deleted(self):
@@ -186,7 +181,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                 fw_id = fw['firewall']['id']
                 with ctx.session.begin(subtransactions=True):
                     fw_db = self.plugin._get_firewall(ctx, fw_id)
-                    fw_db['status'] = const.PENDING_DELETE
+                    fw_db['status'] = nl_constants.PENDING_DELETE
                     ctx.session.flush()
                     res = self.callbacks.firewall_deleted(ctx, fw_id,
                                                           host='dummy')
@@ -217,7 +212,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                 fw_id = fw['firewall']['id']
                 with ctx.session.begin(subtransactions=True):
                     fw_db = self.plugin._get_firewall(ctx, fw_id)
-                    fw_db['status'] = const.PENDING_DELETE
+                    fw_db['status'] = nl_constants.PENDING_DELETE
                     ctx.session.flush()
 
                 with mock.patch.object(
@@ -249,7 +244,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                                                       host='dummy')
                 self.assertFalse(res)
                 fw_db = self.plugin._get_firewall(ctx, fw_id)
-                self.assertEqual(const.ERROR, fw_db['status'])
+                self.assertEqual(nl_constants.ERROR, fw_db['status'])
 
     def test_get_firewall_for_tenant(self):
         tenant_id = 'test-tenant'
@@ -344,9 +339,6 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
         super(TestFirewallPluginBase, self).setUp(fw_plugin=FW_PLUGIN_KLASS)
         fake_notifier.reset()
 
-    def tearDown(self):
-        super(TestFirewallPluginBase, self).tearDown()
-
     def test_create_firewall_routers_not_specified(self):
         """neutron firewall-create test-policy """
         with self.router(name='router1', admin_state_up=True,
@@ -354,7 +346,7 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
             with self.router(name='router2', admin_state_up=True,
                 tenant_id=self._tenant_id):
                 with self.firewall() as fw1:
-                    self.assertEqual(const.PENDING_CREATE,
+                    self.assertEqual(nl_constants.PENDING_CREATE,
                         fw1['firewall']['status'])
 
     def test_create_firewall_routers_specified(self):
@@ -365,7 +357,7 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 tenant_id=self._tenant_id) as router2:
                 router_ids = [router1['router']['id'], router2['router']['id']]
                 with self.firewall(router_ids=router_ids) as fw1:
-                    self.assertEqual(const.PENDING_CREATE,
+                    self.assertEqual(nl_constants.PENDING_CREATE,
                         fw1['firewall']['status'])
 
     def test_create_firewall_routers_present_empty_list_specified(self):
@@ -376,14 +368,14 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 tenant_id=self._tenant_id):
                 router_ids = []
                 with self.firewall(router_ids=router_ids) as fw1:
-                    self.assertEqual(const.INACTIVE,
+                    self.assertEqual(nl_constants.INACTIVE,
                         fw1['firewall']['status'])
 
     def test_create_firewall_no_routers_empty_list_specified(self):
         """neutron firewall-create test-policy --router-ids "" """
         router_ids = []
         with self.firewall(router_ids=router_ids) as fw1:
-            self.assertEqual(const.INACTIVE,
+            self.assertEqual(nl_constants.INACTIVE,
                 fw1['firewall']['status'])
 
     def test_create_second_firewall_on_same_tenant(self):
@@ -395,9 +387,9 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 router_ids = []
                 with self.firewall() as fw1:
                     with self.firewall(router_ids=router_ids) as fw2:
-                        self.assertEqual(const.PENDING_CREATE,
+                        self.assertEqual(nl_constants.PENDING_CREATE,
                             fw1['firewall']['status'])
-                        self.assertEqual(const.INACTIVE,
+                        self.assertEqual(nl_constants.INACTIVE,
                             fw2['firewall']['status'])
 
     def test_create_firewall_admin_not_affected_by_other_tenant(self):
@@ -424,14 +416,16 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 ) as firewall:
                     fw_id = firewall['firewall']['id']
                     res = self.callbacks.set_firewall_status(ctx, fw_id,
-                                                         const.ACTIVE)
+                                                         nl_constants.ACTIVE)
                     data = {'firewall': {'name': name}}
                     req = self.new_update_request('firewalls', data, fw_id)
                     res = self.deserialize(self.fmt,
                                        req.get_response(self.ext_api))
                     attrs = self._replace_firewall_status(attrs,
-                                                      const.PENDING_CREATE,
-                                                      const.PENDING_UPDATE)
+                                                          nl_constants.
+                                                          PENDING_CREATE,
+                                                          nl_constants.
+                                                          PENDING_UPDATE)
                     for k, v in six.iteritems(attrs):
                         self.assertEqual(v, res['firewall'][k])
 
@@ -477,8 +471,10 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                     res = self.deserialize(self.fmt,
                                        req.get_response(self.ext_api))
                     attrs = self._replace_firewall_status(attrs,
-                                                      const.PENDING_CREATE,
-                                                      const.PENDING_UPDATE)
+                                                      nl_constants.
+                                                      PENDING_CREATE,
+                                                      nl_constants.
+                                                      PENDING_UPDATE)
                     for k, v in six.iteritems(attrs):
                         self.assertEqual(v, res['firewall'][k])
 
@@ -497,7 +493,7 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 ) as firewall:
                     fw_id = firewall['firewall']['id']
                     self.callbacks.set_firewall_status(ctx, fw_id,
-                                                   const.ACTIVE)
+                                                   nl_constants.ACTIVE)
                     data = {'firewall': {'shared': True}}
                     req = self.new_update_request(
                         'firewalls', data, fw_id,
@@ -625,6 +621,23 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 self._test_list_resources('firewall', [fwalls],
                                           query_params='description=fw')
 
+    def test_list_firewalls_with_filtering(self):
+        with self.router(name='my_router', admin_state_up=True,
+                         tenant_id=self._tenant_id) as router:
+            router_id = router['router']['id']
+            with self.firewall_policy() as fwp:
+                fwp_id = fwp['firewall_policy']['id']
+                with self.firewall(name='fw1', firewall_policy_id=fwp_id,
+                                   description='fw',
+                                   router_ids=[router_id]) as fwalls:
+                    filter_pattern = None
+                    fw = fwalls['firewall']
+                    for filter_pattern in fw:
+                        query_params = 'fields=%s' % filter_pattern
+                        expect = [{filter_pattern: fw[filter_pattern]}]
+                        self._test_list_resources('firewall', expect,
+                                                  query_params=query_params)
+
     def test_insert_rule(self):
         ctx = context.get_admin_context()
         with self.firewall_rule() as fwr:
