@@ -16,8 +16,6 @@
 import mock
 from neutron.api.v2 import attributes as attr
 from neutron import context
-from neutron import manager
-from neutron.plugins.common import constants as const
 from neutron.tests import fake_notifier
 from neutron.tests.unit.extensions import test_l3 as test_l3_plugin
 from oslo_config import cfg
@@ -29,6 +27,8 @@ from neutron_fwaas.services.firewall import fwaas_plugin_v2
 from neutron_fwaas.tests import base
 from neutron_fwaas.tests.unit.db.firewall.v2 import (
     test_firewall_db_v2 as test_db_firewall)
+from neutron_lib import constants as nl_constants
+from neutron_lib.plugins import directory
 
 extensions_path = neutron_fwaas.extensions.__path__[0]
 
@@ -102,6 +102,7 @@ class TestFirewallRouterPortBase(
         self.saved_attr_map = {}
         for resource, attrs in six.iteritems(attr.RESOURCE_ATTRIBUTE_MAP):
             self.saved_attr_map[resource] = attrs.copy()
+        self.addCleanup(self.restore_attribute_map)
         if not fw_plugin:
             fw_plugin = FW_PLUGIN_KLASS
         service_plugins = {'l3_plugin_name': l3_plugin,
@@ -114,20 +115,14 @@ class TestFirewallRouterPortBase(
 
         self.setup_notification_driver()
 
-        self.l3_plugin = manager.NeutronManager.get_service_plugins().get(
-            const.L3_ROUTER_NAT)
-        self.plugin = manager.NeutronManager.get_service_plugins().get(
-            'FIREWALL_V2')
+        self.l3_plugin = directory.get_plugin(nl_constants.L3)
+        self.plugin = directory.get_plugin('FIREWALL_V2')
         self.callbacks = self.plugin.endpoints[0]
 
     def restore_attribute_map(self):
         # Restore the original RESOURCE_ATTRIBUTE_MAP
         attr.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map
 
-    def tearDown(self):
-        self.restore_attribute_map()
-        super(TestFirewallRouterPortBase, self).tearDown()
-
 
 class TestFirewallCallbacks(TestFirewallRouterPortBase):
 
@@ -146,96 +141,17 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
             ) as fwg:
                 fwg_id = fwg['firewall_group']['id']
                 res = self.callbacks.set_firewall_group_status(ctx, fwg_id,
-                                                         const.ACTIVE,
+                                                         nl_constants.ACTIVE,
                                                          host='dummy')
                 fwg_db = self.plugin.get_firewall_group(ctx, fwg_id)
-                self.assertEqual(const.ACTIVE, fwg_db['status'])
+                self.assertEqual(nl_constants.ACTIVE, fwg_db['status'])
                 self.assertTrue(res)
                 res = self.callbacks.set_firewall_group_status(ctx, fwg_id,
-                                                         const.ERROR)
+                                                         nl_constants.ERROR)
                 fwg_db = self.plugin.get_firewall_group(ctx, fwg_id)
-                self.assertEqual(const.ERROR, fwg_db['status'])
+                self.assertEqual(nl_constants.ERROR, fwg_db['status'])
                 self.assertFalse(res)
 
-    def test_firewall_group_deleted(self):
-        ctx = context.get_admin_context()
-        with self.firewall_policy() as fwp:
-            fwp_id = fwp['firewall_policy']['id']
-            with self.firewall_group(
-                ingress_firewall_policy_id=fwp_id,
-                admin_state_up=test_db_firewall.ADMIN_STATE_UP,
-                do_delete=False
-            ) as fwg:
-                fwg_id = fwg['firewall_group']['id']
-                with ctx.session.begin(subtransactions=True):
-                    fwg_db = self.plugin._get_firewall_group(ctx, fwg_id)
-                    fwg_db['status'] = const.PENDING_DELETE
-
-                observed = self.callbacks.firewall_group_deleted(
-                    ctx, fwg_id, host='dummy')
-                self.assertTrue(observed)
-
-            self.assertRaises(firewall_v2.FirewallGroupNotFound,
-                              self.plugin.get_firewall_group,
-                              ctx, fwg_id)
-
-    def test_firewall_group_deleted_concurrently(self):
-        ctx = context.get_admin_context()
-        alt_ctx = context.get_admin_context()
-
-        _get_firewall_group = self.plugin._get_firewall_group
-
-        def getdelete(context, fwg_id):
-            fwg_db = _get_firewall_group(context, fwg_id)
-            # NOTE(cby): Use a different session to simulate a concurrent del
-            self.plugin.delete_db_firewall_group_object(alt_ctx, fwg_id)
-            return fwg_db
-
-        with self.firewall_policy() as fwp:
-            fwp_id = fwp['firewall_policy']['id']
-            with self.firewall_group(
-                firewall_policy_id=fwp_id,
-                admin_state_up=test_db_firewall.ADMIN_STATE_UP,
-                do_delete=False
-            ) as fwg:
-                fwg_id = fwg['firewall_group']['id']
-                with ctx.session.begin(subtransactions=True):
-                    fwg_db = self.plugin._get_firewall_group(ctx, fwg_id)
-                    fwg_db['status'] = const.PENDING_DELETE
-                    ctx.session.flush()
-
-                with mock.patch.object(
-                    self.plugin, '_get_firewall_group', side_effect=getdelete
-                ):
-                    observed = self.callbacks.firewall_group_deleted(
-                        ctx, fwg_id, host='dummy')
-                    self.assertTrue(observed)
-
-                self.assertRaises(firewall_v2.FirewallGroupNotFound,
-                                  self.plugin.get_firewall_group,
-                                  ctx, fwg_id)
-
-    def test_firewall_group_deleted_not_found(self):
-        ctx = context.get_admin_context()
-        observed = self.callbacks.firewall_group_deleted(
-            ctx, 'notfound', host='hh')
-        self.assertTrue(observed)
-
-    def test_firewall_group_deleted_error(self):
-        ctx = context.get_admin_context()
-        with self.firewall_policy() as fwp:
-            fwp_id = fwp['firewall_policy']['id']
-            with self.firewall_group(
-                firewall_policy_id=fwp_id,
-                admin_state_up=test_db_firewall.ADMIN_STATE_UP,
-            ) as fwg:
-                fwg_id = fwg['firewall_group']['id']
-                observed = self.callbacks.firewall_group_deleted(
-                    ctx, fwg_id, host='dummy')
-                self.assertFalse(observed)
-                fwg_db = self.plugin._get_firewall_group(ctx, fwg_id)
-                self.assertEqual(const.ERROR, fwg_db['status'])
-
 
 class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                              test_l3_plugin.L3NatTestCaseMixin):
@@ -244,8 +160,9 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
         super(TestFirewallPluginBasev2, self).setUp(fw_plugin=FW_PLUGIN_KLASS)
         fake_notifier.reset()
 
-    def tearDown(self):
-        super(TestFirewallPluginBasev2, self).tearDown()
+    @property
+    def _self_context(self):
+        return context.Context('', self._tenant_id)
 
     def test_create_firewall_group_ports_not_specified(self):
         """neutron firewall-create test-policy """
@@ -256,7 +173,7 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                 ingress_firewall_policy_id=fwp_id,
                 egress_firewall_policy_id=fwp_id,
                 admin_state_up=True) as fwg1:
-                self.assertEqual(const.INACTIVE,
+                self.assertEqual(nl_constants.INACTIVE,
                     fwg1['firewall_group']['status'])
 
     def test_create_firewall_group_with_ports(self):
@@ -287,7 +204,7 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                     ingress_firewall_policy_id=fwp_id,
                     egress_firewall_policy_id=fwp_id, ports=fwg_ports,
                     admin_state_up=True) as fwg1:
-                    self.assertEqual(const.PENDING_CREATE,
+                    self.assertEqual(nl_constants.PENDING_CREATE,
                          fwg1['firewall_group']['status'])
 
             self._router_interface_action('remove',
@@ -339,7 +256,7 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                         egress_firewall_policy_id=fwp_id,
                         ports=fwg_ports,
                         admin_state_up=True) as fwg1:
-                        self.assertEqual(const.PENDING_CREATE,
+                        self.assertEqual(nl_constants.PENDING_CREATE,
                             fwg1['firewall_group']['status'])
 
                 self._router_interface_action('remove',
@@ -383,7 +300,7 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                 default_policy=False,
                 ports=fwg_ports,
                 admin_state_up=True) as fwg1:
-                self.assertEqual(const.INACTIVE,
+                self.assertEqual(nl_constants.INACTIVE,
                      fwg1['firewall_group']['status'])
 
             self._router_interface_action('remove',
@@ -431,17 +348,21 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                 default_policy=False,
                 ports=fwg_ports,
                 admin_state_up=True) as fwg1:
-                self.assertEqual(const.INACTIVE,
+                self.assertEqual(nl_constants.INACTIVE,
                      fwg1['firewall_group']['status'])
                 data = {'firewall_group': {'ports': [port_id2, port_id3]}}
                 req = self.new_update_request('firewall_groups', data,
-                                              fwg1['firewall_group']['id'])
+                                              fwg1['firewall_group']['id'],
+                                              context=self._self_context)
                 res = self.deserialize(self.fmt,
                                        req.get_response(self.ext_api))
 
                 self.assertEqual(sorted([port_id2, port_id3]),
                                  sorted(res['firewall_group']['ports']))
 
+                self.assertEqual(nl_constants.INACTIVE,
+                                 res['firewall_group']['status'])
+
             self._router_interface_action('remove',
                 r['router']['id'],
                 s1['subnet']['id'],
@@ -489,7 +410,7 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                     ingress_firewall_policy_id=fwp_id,
                     egress_firewall_policy_id=fwp_id, ports=fwg_ports,
                     admin_state_up=True) as fwg1:
-                    self.assertEqual(const.PENDING_CREATE,
+                    self.assertEqual(nl_constants.PENDING_CREATE,
                          fwg1['firewall_group']['status'])
                     data = {'firewall_group': {'ports': [port_id2, port_id3]}}
                     req = self.new_update_request('firewall_groups', data,
@@ -543,15 +464,16 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                     ingress_firewall_policy_id=fwp_id,
                     egress_firewall_policy_id=fwp_id, ports=fwg_ports,
                     admin_state_up=True) as fwg1:
-                    self.assertEqual(const.PENDING_CREATE,
+                    self.assertEqual(nl_constants.PENDING_CREATE,
                          fwg1['firewall_group']['status'])
 
                     ctx = context.get_admin_context()
                     self.callbacks.set_firewall_group_status(ctx,
-                        fwg1['firewall_group']['id'], const.ACTIVE)
+                        fwg1['firewall_group']['id'], nl_constants.ACTIVE)
                     data = {'firewall_group': {'ports': [port_id2, port_id3]}}
                     req = self.new_update_request('firewall_groups', data,
-                                                  fwg1['firewall_group']['id'])
+                                                  fwg1['firewall_group']['id'],
+                                                  context=self._self_context)
                     res = self.deserialize(self.fmt,
                                            req.get_response(self.ext_api))
                     self.assertEqual(sorted([port_id2, port_id3]),
@@ -589,12 +511,12 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                         ingress_firewall_policy_id=fwp_id,
                         egress_firewall_policy_id=fwp_id, ports=[port_id1],
                         admin_state_up=True) as fwg1:
-                        self.assertEqual(const.PENDING_CREATE,
+                        self.assertEqual(nl_constants.PENDING_CREATE,
                              fwg1['firewall_group']['status'])
 
                         ctx = context.get_admin_context()
                         self.callbacks.set_firewall_group_status(ctx,
-                            fwg1['firewall_group']['id'], const.ACTIVE)
+                            fwg1['firewall_group']['id'], nl_constants.ACTIVE)
                         data = {'firewall_rule': {'name': name}}
                         req = self.new_update_request('firewall_rules', data,
                             fwr['firewall_rule']['id'])
@@ -630,7 +552,7 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                         ingress_firewall_policy_id=fwp_id,
                         egress_firewall_policy_id=fwp_id, ports=[port_id1],
                         admin_state_up=True) as fwg1:
-                        self.assertEqual(const.PENDING_CREATE,
+                        self.assertEqual(nl_constants.PENDING_CREATE,
                              fwg1['firewall_group']['status'])
 
                         data = {'firewall_rule': {'name': name}}
@@ -643,3 +565,101 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                 r['router']['id'],
                 s1['subnet']['id'],
                 None)
+
+    def test_update_firewall_group_with_non_exist_ports(self):
+        """neutron firewall_group create test-policy """
+        with self.router(name='router1', admin_state_up=True,
+                         tenant_id=self._tenant_id) as r, \
+                self.subnet(cidr='30.0.0.0/24') as s:
+            body = self._router_interface_action(
+                'add',
+                r['router']['id'],
+                s['subnet']['id'],
+                None)
+            port_id1 = body['port_id']
+            foo_port_id = 'caef152d-b118-4b9b-bc77-800661bf082d'
+            fwg_ports = [port_id1]
+            with self.firewall_group(
+                    name='test',
+                    default_policy=False,
+                    ports=fwg_ports,
+                    admin_state_up=True) as fwg1:
+                self.assertEqual(nl_constants.INACTIVE,
+                                 fwg1['firewall_group']['status'])
+                data = {'firewall_group': {'ports': [foo_port_id]}}
+                req = self.new_update_request('firewall_groups', data,
+                                              fwg1['firewall_group']['id'])
+                res = self.deserialize(self.fmt,
+                                       req.get_response(self.ext_api))
+                self.assertEqual('PortNotFound',
+                                 res['NeutronError']['type'])
+
+            self._router_interface_action('remove',
+                                          r['router']['id'],
+                                          s['subnet']['id'],
+                                          None)
+
+    def test_update_firewall_group_with_ports_and_polcy(self):
+        """neutron firewall_group create test-policy """
+        with self.router(name='router1', admin_state_up=True,
+                         tenant_id=self._tenant_id) as r,\
+                self.subnet() as s1,\
+                self.subnet(cidr='20.0.0.0/24') as s2:
+
+            body = self._router_interface_action(
+                'add',
+                r['router']['id'],
+                s1['subnet']['id'],
+                None)
+            port_id1 = body['port_id']
+
+            body = self._router_interface_action(
+                'add',
+                r['router']['id'],
+                s2['subnet']['id'],
+                None)
+            port_id2 = body['port_id']
+
+            fwg_ports = [port_id1, port_id2]
+            with self.firewall_rule() as fwr:
+                with self.firewall_policy(
+                        firewall_rules=[fwr['firewall_rule']['id']]) as fwp:
+                    with self.firewall_group(
+                            name='test',
+                            default_policy=False,
+                            ports=fwg_ports,
+                            admin_state_up=True) as fwg1:
+                        self.assertEqual(nl_constants.INACTIVE,
+                             fwg1['firewall_group']['status'])
+                        fwp_id = fwp["firewall_policy"]["id"]
+
+                        data = {'firewall_group': {'ports': fwg_ports}}
+                        req = (self.
+                               new_update_request('firewall_groups', data,
+                                                  fwg1['firewall_group']['id'],
+                                                  context=self._self_context))
+                        res = self.deserialize(self.fmt,
+                                               req.get_response(self.ext_api))
+                        self.assertEqual(nl_constants.INACTIVE,
+                                         res['firewall_group']['status'])
+
+                        data = {'firewall_group': {
+                            'ingress_firewall_policy_id': fwp_id}}
+                        req = (self.
+                               new_update_request('firewall_groups', data,
+                                                  fwg1['firewall_group']['id'],
+                                                  context=self._self_context))
+                        res = self.deserialize(self.fmt,
+                                               req.get_response(self.ext_api))
+                        self.assertEqual(nl_constants.PENDING_UPDATE,
+                                         res['firewall_group']['status'])
+
+                    self._router_interface_action('remove',
+                        r['router']['id'],
+                        s1['subnet']['id'],
+                        None)
+                    self._router_interface_action(
+                        'remove',
+                        r['router']['id'],
+                        s2['subnet']['id'],
+                        None)
